/*
 * Normaliz
 * Copyright (C) 2007-2013  Winfried Bruns, Bogdan Ichim, Christof Soeger
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

#include "libnormaliz.cpp"
#include "integer.cpp"
#include "vector_operations.cpp"
#include "matrix.cpp"
#include "simplex.cpp"
#include "list_operations.cpp"
#include "lineare_transformation.cpp"
#include "sublattice_representation.cpp"
#include "full_cone.cpp"
#include "cone_dual_mode.cpp"
#include "cone.cpp"
#define HILBERT_NON_TEMPLATE
#include "HilbertSeries.cpp"

namespace libnormaliz {

template class Cone<long>;
template class Cone<long long int>;
template class Cone<mpz_class>;

template class Matrix<long>;
template class Matrix<long long int>;
template class Matrix<mpz_class>;

template class Sublattice_Representation<long>;
template class Sublattice_Representation<long long int>;
template class Sublattice_Representation<mpz_class>;

template class Lineare_Transformation<long>;
template class Lineare_Transformation<long long int>;
template class Lineare_Transformation<mpz_class>;

template Lineare_Transformation<long> Transformation(const Matrix<long>& M);
template Lineare_Transformation<long long int> Transformation(const Matrix<long long int>& M);
template Lineare_Transformation<mpz_class> Transformation(const Matrix<mpz_class>& M);

template size_t decimal_length<long>(long);
template size_t decimal_length<long long int>(long long int);
template size_t decimal_length<mpz_class>(mpz_class);

template long gcd<long>(const long& a, const long& b);
template long lcm<long>(const long& a, const long& b);
template long permutations<long>(const size_t& a, const size_t& b);
template long long gcd<long long>(const long long& a, const long long& b);
template long long lcm<long long>(const long long& a, const long long& b);
template long long permutations<long long>(const size_t& a, const size_t& b);
//template mpz_class gcd<mpz_class>(const mpz_class& a, const mpz_class& b);
//template mpz_class lcm<mpz_class>(const mpz_class& a, const mpz_class& b);
template mpz_class permutations<mpz_class>(const size_t& a, const size_t& b);

template ostream& operator<< <long>(ostream& out, const vector<long>& v);
template ostream& operator<< <long long>(ostream& out, const vector<long long>& v);
template ostream& operator<< <mpz_class>(ostream& out, const vector<mpz_class>& v);

template void poly_add_to<long>(vector<long>& a, const vector<long>& b);
template void poly_add_to<long long>(vector<long long>& a, const vector<long long>& b);
template void poly_add_to<mpz_class>(vector<mpz_class>& a, const vector<mpz_class>& b);

template void poly_sub_to<long>(vector<long>& a, const vector<long>& b);
template void poly_sub_to<long long>(vector<long long>& a, const vector<long long>& b);
template void poly_sub_to<mpz_class>(vector<mpz_class>& a, const vector<mpz_class>& b);

template vector<long> poly_mult<long>(const vector<long>& a, const vector<long>& b);
template vector<long long> poly_mult<long long>(const vector<long long>& a, const vector<long long>& b);
template vector<mpz_class> poly_mult<mpz_class>(const vector<mpz_class>& a, const vector<mpz_class>& b);

template void poly_mult_to<long>(vector<long>& a, long d, long e = 1);
template void poly_mult_to<long long>(vector<long long>& a, long d, long e = 1);
template void poly_mult_to<mpz_class>(vector<mpz_class>& a, long d, long e = 1);

template void poly_div<long>(vector<long>& q, vector<long>& r, const vector<long>& a, const vector<long>& b);
template void poly_div<long long>(vector<long long>& q, vector<long long>& r, const vector<long long>& a, const vector<long long>& b);
template void poly_div<mpz_class>(vector<mpz_class>& q, vector<mpz_class>& r, const vector<mpz_class>& a, const vector<mpz_class>& b);

template void remove_zeros<long>(vector<long>& a);
template void remove_zeros<long long>(vector<long long>& a);
template void remove_zeros<mpz_class>(vector<mpz_class>& a);

template vector<long> cyclotomicPoly<long>(long n);
template vector<long long> cyclotomicPoly<long long>(long n);
template vector<mpz_class> cyclotomicPoly<mpz_class>(long n);

template vector<long> coeff_vector<long>(size_t i);
template vector<long long> coeff_vector<long long>(size_t i);
template vector<mpz_class> coeff_vector<mpz_class>(size_t i);

template void linear_substitution<long>(vector<long>& poly, const long& a);
template void linear_substitution<long long>(vector<long long>& poly, const long long& a);
template void linear_substitution<mpz_class>(vector<mpz_class>& poly, const mpz_class& a);

template vector<long> compute_e_vector<long>(vector<long> h_vector, int dim);
template vector<long long> compute_e_vector<long long>(vector<long long> h_vector, int dim);
template vector<mpz_class> compute_e_vector<mpz_class>(vector<mpz_class> h_vector, int dim);

template vector<long> compute_polynomial<long>(vector<long> h_vector, int dim);
template vector<long long> compute_polynomial<long long>(vector<long long> h_vector, int dim);
template vector<mpz_class> compute_polynomial<mpz_class>(vector<mpz_class> h_vector, int dim);
}
